import torch
import numpy as np

def mig_loss_function_multi(output_dict, modal_num, bs, d, p, classes=2):

    '''

    :param output_dict:  bs*C output matrix per value
    :param modal_num: the number of modalities
    :param p: prior
    :param bs: batch size
    :param device: device number

    :return: MIG loss
    '''


    noise = torch.rand(1).cuda() * 0.0001
    normalize_1 = bs
    normalize_2 = len(d[0])

    label_index = np.random.choice(normalize_2, normalize_2, replace=False).astype(np.int16)
    normalize_2 = normalize_2
    # Multiply prior on the denominator
    output_dict[1] = output_dict[1] / (p ** (modal_num-1))

    # Sample joint from mini-batch
    m1 = output_dict[1]
    for i in range(2,modal_num+1):
        m1 = m1 * output_dict[i]
    m1 = torch.log(m1.sum(1) + noise)
    m1 = m1.sum()

    # Sample margin-product from mini-batch
    o1 = output_dict[1][np.array(d[0])[label_index]]
    for i in range(2, modal_num + 1):
        o1 = o1 * output_dict[i][np.array(d[i-1])[label_index]]

    o1 = o1.sum()
    m2 = o1
    return - m1 / normalize_1 + m2 / normalize_2


def get_index_3(bs):

    p = []
    for i in range(0,bs):
        for j in range(0,bs):
            if i == j:
                continue
            for k in range(0,bs):
                if i==k or j==k:
                    continue
                p += [[i,j,k]]
    index_1 = []
    index_2 = []
    index_3 = []

    for i in range(len(p)):
        index_1 += [p[i][0]]
        index_2 += [p[i][1]]
        index_3 += [p[i][2]]
    return [index_1, index_2, index_3]


def hgr_loss_function(output1, output2):
    N = output1.size(0)
    C = output1.size(1)
    I = torch.FloatTensor(np.eye(N), ).cuda()
    output1 = output1 - output1.sum(0)/N
    output2 = output2 - output2.sum(0)/N
    m = (output1 @ output2.transpose(1,0))
    m1 = (m * I).sum() / (N-1)

    cov1 = output1[0].resize(C, 1) @ output1[0].resize(1, C)
    cov2 = output2[0].resize(C, 1) @ output2[0].resize(1, C)
    for i in range(1,N):
        cov1 = cov1 + output1[i].resize(C, 1) @ output1[i].resize(1, C)
        cov2 = cov2 + output2[i].resize(C, 1) @ output2[i].resize(1, C)

    cov1 = cov1 / (N-1)
    cov2 = cov2 / (N-1)

    return -m1 + 0.5 * torch.trace(cov1 @ cov2)