from sklearn.datasets import fetch_20newsgroups
from sklearn.feature_extraction.text import TfidfVectorizer

import numpy as np
import torch
import torch.utils.data
from experiment.main import args
from dataset.CornellReader import view_train,view_test,label_train,label_test,split

class Text(torch.utils.data.Dataset):

    def __init__(self, data, label, train, labeled = True):

        self.m = args.modalities
        self.labeled = labeled
        self.train = train
        if not labeled:

            if train:
                self.data = self.cons_dict(data,self.m)
                self.label = label[split:]

        else:
            if train:
                self.data = self.cons_dict(data,self.m)
                self.label = label[: split]

            else:
                self.data = self.cons_dict(data,self.m)
                self.label = label


    def __len__(self):
        return len(self.label)

    def __getitem__(self, index):

        label = self.label[index]
        data_dict = {i: self.data[i][index] for i in range(1, self.m + 1)}

        return data_dict, label

    def cons_dict(self,data,m):

        t_dict = dict()
        for i in range(m):
            if self.train:
                if self.labeled:
                    t_dict[i + 1] = data[i + 1][:split]
                else:
                    t_dict[i + 1] = data[i + 1][split:]
            else:
                t_dict[i + 1] = data[i + 1]

        return t_dict

class Text_MIG(torch.utils.data.Dataset):

    def __init__(self, data, label):


        self.m = args.modalities
        self.data_dict = self.cons_dict(data,self.m)
        self.label = label[split:]

    def __len__(self):
        return len(self.label)

    def __getitem__(self, index):

        label = self.label[index]
        data_dict = {i:self.data_dict[i][index] for i in range(1,self.m+1)}

        return data_dict, label

    def cons_dict(self,data,m):

        t_dict = dict()
        for i in range(m):
            t_dict[i+1] = data[i+1][split:]
        return t_dict



T_train = Text(data = view_train,label = label_train,train= True)
T_test = Text(data = view_test,label = label_test,train= False)
U_train = Text(data = view_train,label = label_train,train= True,labeled=False)
U_MIG = Text_MIG(data = view_train,label = label_train)


dataloader = {'train':torch.utils.data.DataLoader(dataset = T_train, batch_size = 32),
'test':torch.utils.data.DataLoader(dataset = T_test, batch_size = 32),
'mig':torch.utils.data.DataLoader(dataset = U_MIG, batch_size = 32),
'vat':torch.utils.data.DataLoader(dataset = U_MIG, batch_size = 32)}
