# -*- coding: utf-8 -*-
from __future__ import print_function
import argparse
import torch.nn as nn
import torch

# seed = 123
torch.backends.cudnn.benchmark = False
torch.backends.cudnn.deterministic = True
# torch.manual_seed(seed)
# torch.cuda.manual_seed(seed)
# torch.cuda.manual_seed_all(seed)

import os
from util.utils import *

def load_pytorch_model(model, path, opt):
    from collections import OrderedDict  # 产生有序字典
    checkpoint = torch.load(path, map_location=lambda storage, loc: storage)  # 加载tar类型的model
    state_dict = checkpoint['state_dict']  # 得到模型的状态参数字典
    if len(opt.cuda_id[0]) >= 1:  # 如果使用GPU需要加载到CPU上面测试
        new_state_dict = OrderedDict()
        for k, v in state_dict.items():
            # print k
            if k[:7] == 'module.':
                name = k[7:]  # remove `module.`,分布式GPU训练的模型的key值会多了'module.'
                new_state_dict[name] = v
            else:
                new_state_dict[k] = v
        # load params
        try:
            model.load_state_dict(new_state_dict)
        except:
            model.load_state_dict(new_state_dict, strict=False)
    else:
        model.load_state_dict(state_dict)

    return model


def get_opts():
    parser = argparse.ArgumentParser(description='PyTorch attribute learning')
    parser.add_argument('--batchSize', type=int, default=4, help='training batch size')
    parser.add_argument('--testBatchSize', type=int, default=1, help='training batch size')
    parser.add_argument('--nb_epochs', type=int, default=100, help='training batch size')
    parser.add_argument('--cuda_id', type=str, default='0', nargs='*', help='cuda id. Default=random')
    parser.add_argument('--input_num', type=int, default=1, help='input shape number')
    parser.add_argument('--crop_size', type=int, default=64, help='input shape number')
    parser.add_argument('--WC_num', type=int, default=1, help='WC number')
    parser.add_argument('--WC1', type=int, default=600, help='training batch size')
    parser.add_argument('--WC2', type=int, default=40, help='training batch size')
    parser.add_argument('--early', type=int, default=10, help='earlyStop')
    parser.add_argument('--lw_class', type=float, default=1.0, help='training batch size')
    parser.add_argument('--pall', type=int, default=1, help='training batch size')
    parser.add_argument('--bn', type=int, default=1, help='training batch size')
    parser.add_argument('--lr', type=float, default=0.002, help='training batch size')
    parser.add_argument('--model', type=str, default='ISBI_large', help='cuda id. Default=random')
    parser.add_argument('--pool', type=int, default=1, help='training batch size')
    parser.add_argument('--fc1', type=int, default=128, help='training batch size')
    parser.add_argument('--dp1', type=float, default=0.3, help='training batch size')
    parser.add_argument('--isseg', type=int, default=1, help='training batch size')
    parser.add_argument('--class_num', type=int, default=1, help='training batch size')
    parser.add_argument('--regularizer', type=float, default=0.001, help='training batch size')
    parser.add_argument('--alpha', type=float, default=0.75, help='training batch size')
    parser.add_argument('--gamma', type=float, default=2.0, help='training batch size')
    parser.add_argument('--bias', type=int, default=1, help='training batch size')
    parser.add_argument('--sigmoid', type=int, default=0, help='training batch size')
    parser.add_argument('--nb_classes', type=int, default=2, help='training batch size')
    parser.add_argument('--fc_num', type=int, default=1, help='training batch size')
    parser.add_argument('--mul_max', type=float, default=2.0, help='training batch size')
    parser.add_argument('--rand_mask', type=int, default=0, help='training batch size')
    parser.add_argument('--rand_size', type=int, default=16, help='training batch size')
    parser.add_argument('--noise', type=int, default=0, help='training batch size')
    parser.add_argument('--aug', type=int, default=0, help='training batch size')
    parser.add_argument('--flip', type=int, default=0, help='training batch size')
    parser.add_argument('--shift', type=int, default=2, help='training batch size')
    parser.add_argument('--transpose', type=int, default=0, help='training batch size')
    parser.add_argument('--val_epoch', type=int, default=5, help='training batch size')
    parser.add_argument('--optimizer', type=str, default='Adam', help='optimizer')
    parser.add_argument('--merge', type=int, default=0, help='is merge label')
    parser.add_argument('--scaling', type=int, default=0, help='dataset scaling data aug')
    parser.add_argument('--iter_l', type=int, default=0, help='iterative learning for multi-task')
    parser.add_argument('--lr_decay', type=float, default=0.1, help='lr decay')
    parser.add_argument('--lr_controler', type=int, default=100, help='lr_controler decrease lr')
    parser.add_argument('--model_k', type=int, default=32, help='for attribute hidden length')
    parser.add_argument('--model_f', type=int, default=512, help='for feature length')
    parser.add_argument('--act', type=str, default='Sigm', help='activation function type')
    parser.add_argument('--concat', type=int, default=0, help='is concat segmentation')
    parser.add_argument('--gc_num', type=int, default=2, help='number of gcn layers')
    parser.add_argument('--kernel_mul', type=int, default=1, help='gcn kernel para')
    parser.add_argument('--graph_type', type=int, default=2664, help='graph type')
    parser.add_argument('--graph_norm', type=int, default=1, help='graph type')
    parser.add_argument('--graph_sym', type=int, default=1, help='graph sym')
    parser.add_argument('--multi-scale', type=int, default=2664, help='graph type')
    parser.add_argument('--sampler', type=int, default=0, help='balanced sampler')
    parser.add_argument('--K', type=int, default=1, help='chepy ranking ')
    parser.add_argument('--attr_id', type=int, default=8, help='denotes attribute id')
    parser.add_argument('--resume', type=int, default=0, help='resume training')
    parser.add_argument('--norm', type=int, default=0, help='minus 0.5 divided by 0.5')
    parser.add_argument('--in_fea_num', type=int, default=1, help='for 153 network, each node feature num')
    parser.add_argument('--TopK', type=int, default=3, help='used only in pareto set')
    parser.add_argument('--begin_epoch', type=int, default=50, help='pareto_set after epoch')
    # start epoch?
    parser.add_argument('--threads', type=int, default=16, help='pareto_set after epoch')
    parser.add_argument('--layer_num', type=int, default=4, help='pareto_set after epoch')
    parser.add_argument('--inference', type=int, default=0, help='inference only')
    parser.add_argument('--pair_type', type=int, default=1, help='selected pairs')
    parser.add_argument('--m_concat', type=int, default=0, help='model concat')
    parser.add_argument('--DN', type=int, default=0, help='DN')
    parser.add_argument('--rotation', type=int, default=1, help='rotation')
    parser.add_argument('--cost', type=int, default=0, help='cost')
    parser.add_argument('--pareto', type=int, default=0, help='used pareto')
    parser.add_argument('--Nb', type=int, default=20, help='number of iteration')
    parser.add_argument('--each_epoch', type=int, default=10, help='epochs in each iteration')
    parser.add_argument('--v', type=float, default=0.1, help='shranking parameter')
    parser.add_argument('--n_fold', type=int, default=1, help='n_fold')
    parser.add_argument('--train_fold', type=int, default=0, help='train_fold')
    parser.add_argument('--base_dir', type=str, default='/data2/botong', help='/data2/botong')
    parser.add_argument('--policy', type=str, default='cr', help='policy of unsure data')
    parser.add_argument('--testing', type=int, default=0, help='testing only with model already trained')
    parser.add_argument('--old_type', type=int, default=0, help='Lambda old type')
    parser.add_argument('--seed', type=int, default=123, help='random seed')
    parser.add_argument('--test_epoch', type=int, default=300, help='testing epochs')
    parser.add_argument('--shuffle_id', type=int, default=0, help='shuffle_id')
    parser.add_argument('--mig', type=int, default=0, help='mig or not')
    parser.add_argument('--hgr', type=int, default=0, help='hgr or not')
    parser.add_argument('--s', type=int, default=0, help='hgr or not')
    parser.add_argument('--n', type=int, default=0, help='hgr or not')
    parser.add_argument('--file_seed', type=int, default=0, help='seed of txt')
    parser.add_argument('--a', type=float, default=1., help='mig weights')
    parser.add_argument('--r', type=float, default=1., help='label rate')
    parser.add_argument('--feature', type=int, default=0, help='get feature')
    parser.add_argument('--verbose', type=int, default=1, help='verbose temporary results')
    parser.add_argument('--inference_AD', type=int, default=0, help='inference_AD only')
    parser.add_argument('--alpha_vec', type=int, default=0, help='learn a vector alpha')

    # opt.regularizer, opt.isseg, opt.class_num, opt.lw_class
    opt = parser.parse_args()
    # os.environ["CUDA_VISIBLE_DEVICES"] = opt.cuda_id[0]
    cuda = True
    if not torch.cuda.is_available():
        raise Exception("No GPU found, please run without --cuda")

    # opt.threads = 1
    # opt.n_fold = 1
    # opt.base_dir = '/data2/botong'
    # opt.train_fold = 0
    opt.xi = 0
    # opt.inference = 0
    opt.testBatchSize = opt.batchSize

    if opt.layer_num == 4:
        opt.block_config = (6, 12, 24, 12)
    elif opt.layer_num == 3:
        opt.block_config = (6, 12, 24)
    elif opt.layer_num == 2:
        opt.block_config = (6, 12)
    elif opt.layer_num == 6:
        opt.block_config = (3, 6, 12)
    elif opt.layer_num == 7:
        opt.block_config = (4, 6, 12)

    if opt.concat == 1:
        opt.channel_num = 2
    else:
        opt.channel_num = 1

    if opt.graph_type == 153:
        opt.edges = 153
    else:
        opt.edges = opt.class_num

    opt.class_config = [2] # num_class
    opt.class_num = 1

    return opt


def get_model(opt):
    from models.densenet_att_single import DenseNet_l3
    model = DenseNet_l3(sample_size=opt.crop_size, sample_duration=opt.crop_size,
                        inchannel=opt.channel_num, drop_rate=opt.dp1,
                        num_classes=opt.class_num, classes_config=opt.class_config,
                        gc_num=opt.gc_num, kernel_mul=opt.kernel_mul, K=opt.K,
                        in_fea_num=opt.in_fea_num, edges=opt.edges,
                        block_config=opt.block_config)

    return model
