import numpy as np
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt
import time

class Optimizer():

    def __init__(self,alpha,beta):

        self.alpha = alpha
        self.beta = beta
        self.eps  = 1
        self.x = np.array([-2.,2.])

    def opt_conjugate(self, mode):
        value_collect = []
        iteration_collect = []
        time_collect = []
        x = self.x
        iteration = 0

        g_old = self.Gradient(x)
        dx = -g_old
        begin_time = time.time()
        while np.sqrt((g_old ** 2).sum()) > self.eps:
            print("gradient:", np.sqrt((g_old ** 2).sum()), "value:", np.log(self.forward(x) - 0))
            value_collect += [np.log(self.forward(x)-0)]
            iteration_collect += [iteration]
            time_collect += [(time.time() - begin_time)]
            upper = 0.001
            lower = 0
            cnt = 0
            u_value = self.forward(x + upper * dx)
            l_value = self.forward(x + lower * dx)
            # exact line search
            while cnt <= 10:
                cnt += 1
                if u_value > l_value:
                    lower = (upper + lower)/2
                    l_value = self.forward(x + lower * dx)
                else:
                    upper = (upper + lower)/2
                    u_value = self.forward(x + upper * dx)

            t = (upper + lower)/2
            x = x + t * dx

            g_new = self.Gradient(x)

            if mode == 0:
                beta = self.Hestenes_Stiefel(g_new,g_old,dx)
            elif mode == 1:
                beta = self.Polak_Ribiere(g_new,g_old)
            elif mode == 2:
                beta = self.Fletcher_Reeves(g_new,g_old)

            dx = -g_new + beta * dx
            g_old = g_new

            iteration += 1


        iteration_collect = np.array(iteration_collect)
        value_collect = np.array(value_collect)
        time_collect = np.array(time_collect)
        plt.plot(time_collect, value_collect,label = 'Fletcher_Reeves')
        plt.title('Figure 1')

        plt.xlabel('Running time')
        plt.ylabel('Value')
    def opt_damped(self):
        value_collect = []
        iteration_collect = []
        time_collect = []
        x = self.x
        eta = 5e-4
        iteration = 0
        dx = self.Guass_Newton_descent_direction(x)
        g = self.Gradient(x)
        begin_time = time.time()
        while np.sqrt((g ** 2).sum()) > self.eps:
            value_collect += [np.log(self.forward(x)-0)]
            iteration_collect += [iteration]
            time_collect += [int(time.time() - begin_time)]
            t = 1

            #back-tracking
            if np.sqrt((g ** 2).sum()) < eta:
                alpha = 1
            else:
                alpha = self.alpha
            while self.forward(x + t * dx) > self.forward(x) + alpha*t* g.T.dot(dx):
                t *= self.beta
            print(t)
            x = x + t * dx

            dx = self.Guass_Newton_descent_direction(x)
            g = self.Gradient(x)
            iteration += 1
            print("gradient:", np.sqrt((g ** 2).sum()), "value:", np.log(self.forward(x)-0))

        iteration_collect = np.array(iteration_collect)
        value_collect = np.array(value_collect)
        time_collect = np.array(time_collect)
        plt.plot(time_collect, value_collect,label = 'Damped Newton Method with alpha = '+str(self.alpha)+',beta ='+str(self.beta)+',eta=1e-3')
        plt.title('Figure 1')

        plt.xlabel('Running time')
        plt.ylabel('Value')

    def opt_steepest(self):
        value_collect = []
        iteration_collect = []
        time_collect = []
        x = self.x
        iteration = 0
        dx = self.Guass_Newton_descent_direction(x)
        g = self.Gradient(x)
        begin_time = time.time()
        while np.sqrt((g ** 2).sum()) > self.eps:
            value_collect += [np.log(self.forward(x)-0)]
            iteration_collect += [iteration]
            time_collect += [(time.time() - begin_time)]
            t = 1

            #back-tracking
            while self.forward(x + t * dx) > self.forward(x) + self.alpha*t* g.T.dot(dx):
                t *= self.beta

            x = x + t * dx

            dx = self.Guass_Newton_descent_direction(x)
            g = self.Gradient(x)
            iteration += 1
            print("gradient:", np.sqrt((g ** 2).sum()), "value:", np.log(self.forward(x)-0))

        iteration_collect = np.array(iteration_collect)
        value_collect = np.array(value_collect)
        time_collect = np.array(time_collect)
        plt.plot(time_collect, value_collect,label = 'Steepest descent with alpha = '+str(self.alpha)+',beta ='+str(self.beta))
        plt.title('Figure 1')

        plt.xlabel('Running time')
        plt.ylabel('Value')

    def opt_NG(self):
        value_collect = []
        iteration_collect = []
        time_collect = []
        x = self.x

        dx = self.Guass_Newton_descent_direction(x)
        g = self.Gradient(x)
        iteration = 0
        begin_time = time.time()

        while np.sqrt((g ** 2).sum()) > self.eps:
            value = self.forward(x)

            value_collect += [np.log(value-0)]
            iteration_collect += [iteration]
            time_collect += [(time.time()-begin_time)]
            x = x + dx

            iteration += 1
            dx = self.Guass_Newton_descent_direction(x) # update descent direction
            g = self.Gradient(x) # update gradient
            print("gradient:", np.sqrt((g ** 2).sum()), "value:", np.log(value-0))
        plt.plot(time_collect, value_collect,label='Guass-Newton')

    def Hestenes_Stiefel(self,g_new,g_old,d):

        return g_new.dot(g_new - g_old) / d.dot(g_new-g_old)

    def Polak_Ribiere(self,g_new,g_old):

        return g_new.dot(g_new - g_old) / g_old.dot(g_old)

    def Fletcher_Reeves(self,g_new,g_old):

        return g_new.dot(g_new) / g_old.dot(g_old)

    def Gradient(self,x):

        G = np.zeros(2)
        G[0] = 400*np.power(x[0],3) - 400 * x[0] * x[1] + 2*x[0] - 2
        G[1] = 200 * (x[1]-x[0]*x[0])
        return G

    def forward(self,x):

        return 100* ((x[1] - x[0]*x[0])**2) + (1-x[0]) **2

    def Hessian(self,x):

        H = np.zeros((2,2))
        H[0,0] = 1200 * x[0] * x[0] + 2 - 400 * x[1]
        H[1,0] = H[0,1] = -400 * x[0]
        H[1,1] = 200

        return H

    def Guass_Newton_descent_direction(self,x):
        H = self.Hessian(x)
        g = self.Gradient(x)

        #negative ！
        return -np.linalg.inv(H).dot(g)




alpha_box = [0.1,0.3,0.5,0.7,0.9]

beta_box = [0.7]



Opt_ = Optimizer(alpha=0.5, beta = 0.7)
#Opt_.opt_steepest()
#Opt_.opt_NG()

#Opt_.opt_NG()
#Opt_.opt_damped()
Opt_.opt_conjugate(mode=1)
plt.title('Figure 1')
plt.xlabel('Running time')
plt.ylabel('Value')
plt.legend()


#plt.show()
plt.savefig("./mode_1.jpg")

